#!/bin/sh
######################################################################
#
# Tivoli Enterprise Data Warehouse, v 1.1.0
# (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#
#######################################################################
#
#   File Information: %W%
#   Revision: %I%
#   Date: %G% %U%
#
# Desc:  This is the template for WPack application after scripts.
#        It is intended to be used by WPack's that have additional
#        activities needed during the WPack installation, but that
#        there is no default logic in the WPack installer to handle.
#        There is a hook to execute a Core after script that is
#        required to allow dynamic patching of the WPack install
#        process.  Do not remove this hook.
#
#       NOTE - This after script only runs on the Control Server
#              and not on other distributed hosts.
#
# Command Line Args:
#       1 = the path and name of the TWH Application config file
#       2 = Application working directory or install base directory
#               $TWH_TOPDIR/apps/ava_code_for_app/vapp_version
#               (look at install tree)
#
# Required Config File:
#       twh_app_install.cfg
#
# Installation Logs (Default names):
#       A log will be generated for this scirpt called
#       <ava>_after.sh.log
#
#
# Required Env Vars:
#       TWH_TOPDIR
#
# Return Codes (Only!):
#       0 = Success
#       255 = Failure
#
# Interpreter Required:
#       NT/2000 = BASH
#       Unix    = SH (Bourne Shell)
#
#
# Message Types:
#       (I) - Informational
#       (W) - Warning
#       (F) - Fatal
#
# Install Steps Required Before this Script Executes:
#        * TWH Core tools components must be successfully installed
#
# Note - Caution needs to be taken for NT paths that have a space.
#        This is a limitation that requires double quoting around all
#        variables that contain a path.  Testing should be done to
#        ensure this script runs with a space in any of the paths
#        that are used.  For example "c:/program files".
#
######################################################################
#

echo "                ** After Script Log **"
echo ""
echo "****************************************"
echo "* Application After Script Start point *"
echo "****************************************"


# Set parms
echo "      Parm 1 = $1"; TWH_CONFIG_FILE="$1"; export TWH_CONFIG_FILE
echo "      Parm 2 = $2"; APP_WORK_DIR="$2"; export APP_WORK_DIR
echo ""

# TWH_TOPDIR needs to always be in the environment
if [ -z "$TWH_TOPDIR" ]; then
  echo "      (F) Could not locate Env var TWH_TOPDIR"
  exit 255
fi

# Execute startup driver for afterscript
. "$TWH_TOPDIR"/install/bin/twh_core_startup_after.sh
exec_startup_driver;

#####################################################
######## Add application code below here ONLY #######
#####################################################
#
create_new_patch_directory()
{
echo "==> Adding Patch Directories"

# Add a directory for marker files to be kept
if [ ! -d "$TWH_TOPDIR"/apps/bwm/.installed  ]; then
 mkdir -p "$TWH_TOPDIR"/apps/bwm/.installed
  if [ $? -ne 0 ]; then
    echo "      (F) Failed to create the (.installed) directory"
    exit 81
  fi
  echo "      (I) Created .installed directory"
else
  echo "      (I) .installed directory exists"
fi
}

add_patch_v5104()
{
        echo "==> Adding Patch 5.1.0.4"
	if [ -f "$TWH_TOPDIR"/.twh_core_v1.1.2.marker ]
	then
		echo "      (I) TDW core 1.2 not detected"
		touch  "$TWH_TOPDIR"/apps/bwm/.installed/.twh_bwm_v5.1.0.4.marker
	else
		echo "      (I) TDW core 1.2 detected"
		twh_setwepver BWM 5.1.0 5.1.0.4
	fi
	if [ $? -ne 0 ]; then
            echo "==> (F) Failed to add patch 5.1.0.4"
            echo "        Error is:"
            echo "        $err_txt"
            echo "        Exiting..."
            exit 255
	else
	    echo "      (I) Patch 5.1.0.4 successfully added"
	fi
}

set_bwm_program_vars()
{
BWM_MISC_DIR="$APP_WORK_DIR/misc"     # path to bash script to be copied
tmp_sql_file="bwm_tmp_sql.sql"        # name of the temporary sql file

TRUE=1
FALSE=0

return 0;
}

# Prevent from running on wrong host
block_exec_by_host()
{
echo "==> Validating execution host"

# for this prototype, only allow this work to be done on the Control Server
if [ $ADMIN_CHECKED -ne $TRUE ]; then
  echo "      (I) Nothing to be done for this host."
  echo "          Returning control to main executing thread."
  exit 0;
fi

echo "      (I) Execution host is valid.  Program will continue."
return 0;
}

set_db2_params_and_create_tables()
{
echo "==> Starting to set DB2 parameters"


# is the CDW and mart local?
if [ "$CDW_CHECKED" = "$TRUE" ]; then
  echo "    Local CDW..."

  err_txt=`$DB2EXEC update db cfg for "$TWH_CDW_DB" using APP_CTL_HEAP_SZ 512 2>&1 `
  if [ $? -ne 0 ]; then
    echo "==> (F) Failed to set APP_CTL_HEAP_SZ for $TWH_CDW_DB database"
    echo "        Error is:"
    echo "        $err_txt"
    echo "        Exiting..."
    exit 255
  fi

  tmp_sql_drop_bwm_bp="drop tablespace BWM32K;
  drop tablespace TEMP_BWM32K;
  drop tablespace BWMUSER32K;"

  tmp_sql_bp="CREATE BUFFERPOOL BWMBP32K SIZE 10000 PAGESIZE 32K;"


  # Create a 32K page size bufferpool

  # Drop the sys temp tablespace then the 32K page size bufferpool
  echo "      (I) Drop sys temp tablespace and bufferpool"
  echo "$tmp_sql_drop_bwm_bp" > "$TMPDIR/$tmp_sql_file"
  gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CDW_DB" $FALSE;
  rm -f "$TMPDIR/$tmp_sql_file"

  # Create it
  echo "      (I) Create bufferpool"
  echo "$tmp_sql_bp" > "$TMPDIR/$tmp_sql_file"
  gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
  rm -f "$TMPDIR/$tmp_sql_file"

  # Bounce DWC services and DB2
  echo "==> Stopping DWC Services and DB2"
  echo "      (I) Stopping the DWC V7 kernel service, ignore error if DB2 is V8"
  err_txt=`net stop vwkernel 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  echo "      (I) Stopping the DWC V7 logger service, ignore error if DB2 is V8"
  err_txt=`net stop vwlogger 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 

  echo "      (I) Stopping the DWC V8 kernel service, ignore error if DB2 is V7"
  err_txt=`net stop db2dwserver 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  echo "      (I) Stopping the DWC V8 logger service, ignore error if DB2 is V7"
  err_txt=`net stop db2dwlogger 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 

  echo "      (I) Stopping DB2"
  err_txt=`$DB2CMD db2stop force 2>&1 `
  if [ $? -ne 0 ]; then
    blah=`echo $err_txt | grep -i $sql_db_not_started `
    if [ $? -eq 0 ]; then
      :;                  # If db not started that is OK
    else
      echo "==> (F) Failed to stop DB2."
      echo "        Error is:"
      echo "        $err_txt"
      echo "        Exiting..."
      exit 255
    fi
  else
    echo "        Message is:"
    echo "        $err_txt"
    echo "        Continuing..."
  fi

  # Start DB2 and DWC services
  echo "==> Starting DB2 and DWC Services"
  echo "      (I) Starting DB2"
  err_txt=`$DB2CMD db2start 2>&1 `
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  # Note: ignore errors from db2start
  echo "      (I) Starting the DWC V7 logger service, ignore error if DB2 is V8"
  err_txt=`net start vwlogger 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  echo "      (I) Starting the DWC V7 kernel service, ignore error if DB2 is V8"
  err_txt=`net start vwkernel 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  echo "      (I) Starting the DWC V8 logger service, ignore error if DB2 is V7"
  err_txt=`net start db2dwlogger 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 
  echo "      (I) Starting the DWC V8 kernel service, ignore error if DB2 is V7"
  err_txt=`net start db2dwserver 2>&1`
  echo "        Message is:"
  echo "        $err_txt"
  echo "        Continuing..."
  echo "    " 

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  # Have to do this again in case db2bp thread was forced
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  # create regular tablespace that uses 32K page size bufferpool
  tmp_sql_bwmrg="CREATE TEMPORARY TABLESPACE TEMP_BWM32K PAGESIZE 32K MANAGED BY SYSTEM USING ('BWM_32K_2') BUFFERPOOL BWMBP32K DROPPED TABLE RECOVERY OFF;
CREATE USER TEMPORARY TABLESPACE BWMUSER32K PAGESIZE 32 K MANAGED BY SYSTEM USING ('BWMUSER32K') EXTENTSIZE 32 OVERHEAD 24.1 PREFETCHSIZE 32 TRANSFERRATE 0.9 BUFFERPOOL BWMBP32K;"


  # Create it
  echo "      (I) Create tablespaces"
  echo "$tmp_sql_bwmrg" > "$TMPDIR/$tmp_sql_file"
  gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_CONNECT;
  gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
  rm -f "$TMPDIR/$tmp_sql_file"

# We have a remote CDW
else
  echo "    Remote CDW..."

  # Prep vars
  sql_drop_users="disconnect all;
detach;"

  # Build and run script to drop users
  echo "      (I) Dropping applications users from CDW"
  echo "$sql_drop_users" > "$TMPDIR/$tmp_sql_file"
  gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_ATTACH;
  gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
  rm -f "$TMPDIR/$tmp_sql_file"

  # drop users is asynchronous - pause
  sleep 10

  # Test a connection
  echo "      (I) Testing the DB connection"
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi

  # Have to do this again in case db2bp thread was forced
  err_txt=`$DB2EXEC_LOG connect to "$TWH_CDW_DB" user "$COPT_CDW_DB2USER" using "$COPT_CDW_DB2PASS" 2>&1 `
  if [ $? -ne 0 ]; then
    echo "      (F) Unable to connect to database ($TWH_CDW_DB)."
    echo "          Error is:"
    echo "          $err_txt"
    echo "          Exiting..."
    exit 255
  fi


  # Build and run script to drop users (Again)
  echo "      (I) Dropping applications users from CDW - again"
  echo "$sql_drop_users" > "$TMPDIR/$tmp_sql_file"
  gr_build_exec_script "$TWH_CDW_DB" "$TMPDIR" "$tmp_sql_file" $COPT_CDW_TYPE $CONN_TYPE_ATTACH;
  gr_exec_sql_script "$TWH_CDW_DB" $TRUE;
  rm -f "$TMPDIR/$tmp_sql_file"
fi

echo "      (I) Done setting DB parameters"
return 0;
}



################
# Main section #
################
echo "==> Processing main WPack routines"
echo ""

  set_bwm_program_vars;
  block_exec_by_host;
  set_db2_params_and_create_tables;
  create_new_patch_directory;
  add_patch_v5104;

echo ""
echo "      (I) Done processing main WPack routines"
#####################################
## Stop adding application code #####
#####################################
exec_core_after;		# DO NOT MOVE OR REMOVE!


echo ""
echo "==> Completed executing application after script"
exit 0

